@@ -92,7 +92,8 @@ static void e_cal_backend_file_finalize (GObject *object);
 
 static ECalBackendSyncClass *parent_class;
 
-
+static ECalBackendSyncStatus
+e_cal_backend_file_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj);
 
 /* g_hash_table_foreach() callback to destroy a ECalBackendFileObject */
 static void
@@ -689,6 +690,8 @@ notify_removals_cb (gpointer key, gpointer value, gpointer data)
 	if (!g_hash_table_lookup (context->new_uid_hash, uid)) {
 		icalcomponent *old_icomp;
 		gchar *old_obj_str;
+		ECalComponent *comp; 
+		ECalComponentId *id;
 
 		/* Object was removed */
 
@@ -699,8 +702,15 @@ notify_removals_cb (gpointer key, gpointer value, gpointer data)
 		old_obj_str = icalcomponent_as_ical_string (old_icomp);
 		if (!old_obj_str)
 			return;
+		
+		comp = e_cal_component_new_from_string (old_obj_str);
+		id = e_cal_component_get_id (comp);
+		
 
-		e_cal_backend_notify_object_removed (context->backend, uid, old_obj_str, NULL);
+		e_cal_backend_notify_object_removed (context->backend, id, old_obj_str, NULL);
+	
+		e_cal_component_free_id (id);
+		g_object_unref (comp);
 	}
 }
 
@@ -886,6 +896,9 @@ e_cal_backend_file_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 			status = create_cal (cbfile, str_uri);
 	}
 
+	if (status == GNOME_Evolution_Calendar_Success)
+		e_cal_backend_file_add_timezone (backend, cal, (const char *) icaltimezone_get_tzid (priv->default_zone));
+
 	g_free (str_uri);
 
 	return status;
@@ -1166,20 +1179,14 @@ e_cal_backend_file_set_default_timezone (ECalBackendSync *backend, EDataCal *cal
 {
 	ECalBackendFile *cbfile;
 	ECalBackendFilePrivate *priv;
-	icaltimezone *zone;
 
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
 
 	g_return_val_if_fail (priv->icalcomp != NULL, GNOME_Evolution_Calendar_NoSuchCal);
 
-	/* Look up the VTIMEZONE in our icalcomponent. */
-	zone = icalcomponent_get_timezone (priv->icalcomp, tzid);
-	if (!zone)
-		return GNOME_Evolution_Calendar_ObjectNotFound;
-
 	/* Set the default timezone to it. */
-	priv->default_zone = zone;
+	priv->default_zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -1873,11 +1880,26 @@ e_cal_backend_file_modify_object (ECalBackendSync *backend, EDataCal *cal, const
 			priv->comp = g_list_remove (priv->comp, recurrence);
 			obj_data->recurrences_list = g_list_remove (obj_data->recurrences_list, recurrence);
 			g_hash_table_remove (obj_data->recurrences, rid);
+		} else {
+			char *old, *new;
+
+			old = e_cal_component_get_as_string (obj_data->full_object);
+
+			new = e_cal_component_get_as_string (comp);
+			e_cal_backend_notify_object_modified (E_CAL_BACKEND (backend), old, new);
+
+			if (old_object)
+				*old_object = old;
+			else
+				g_free (old);
+			g_free (new);
 		}
 
 		/* add the detached instance */
+		icalcomponent_set_recurrenceid (e_cal_component_get_icalcomponent (comp),
+						icaltime_from_string (rid));
 		g_hash_table_insert (obj_data->recurrences, 
-				     g_strdup (rid),
+				     g_strdup (e_cal_component_get_recurid_as_string (comp)),
 				     comp);
 		icalcomponent_add_component (priv->icalcomp,
 					     e_cal_component_get_icalcomponent (comp));
@@ -1997,27 +2019,6 @@ remove_instance (ECalBackendFile *cbfile, ECalBackendFileObject *obj_data, const
 	cbfile->priv->comp = g_list_prepend (cbfile->priv->comp, obj_data->full_object);
 }
 
-static char *
-get_object_string_from_fileobject (ECalBackendFileObject *obj_data, const char *rid)
-{
-	ECalComponent *comp = obj_data->full_object;
-	char *real_rid;
-
-	if (!rid) {
-		return e_cal_component_get_as_string (comp);
-	} else {
-		if (g_hash_table_lookup_extended (obj_data->recurrences, rid, (void **)&real_rid, (void **)&comp))
-			return e_cal_component_get_as_string (comp);
-		else {
-			/* TODO remove this once we delete an instance from master object through
-			   modify request by setting exception */
-			return e_cal_component_get_as_string (comp);
-		}	
-	}
-
-	return NULL;
-}
-
 /* Remove_object handler for the file backend */
 static ECalBackendSyncStatus
 e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
@@ -2030,7 +2031,6 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	ECalBackendFileObject *obj_data;
 	ECalComponent *comp;
 	RemoveRecurrenceData rrdata;
-	const char *recur_id = NULL;
 
 	cbfile = E_CAL_BACKEND_FILE (backend);
 	priv = cbfile->priv;
@@ -2043,35 +2043,37 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 	obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 	if (!obj_data)
 		return GNOME_Evolution_Calendar_ObjectNotFound;
-	
-	if (rid && *rid)
-		recur_id = rid;
-	
+
 	comp = obj_data->full_object;
 
 	switch (mod) {
 	case CALOBJ_MOD_ALL :
-		*old_object = get_object_string_from_fileobject (obj_data, recur_id);
+		if (comp) {
+			*old_object = e_cal_component_get_as_string (comp);
+		} else {
+			char *real_rid;
+
+			if (g_hash_table_lookup_extended (obj_data->recurrences, rid, (void **)&real_rid, (void **)&comp))
+				*old_object = e_cal_component_get_as_string (comp);
+		}
+
 		remove_component (cbfile, uid, obj_data);
 
 		*object = NULL;
 		break;
 	case CALOBJ_MOD_THIS :
-		if (!recur_id) {
-			*old_object = get_object_string_from_fileobject (obj_data, recur_id);
+		*old_object = e_cal_component_get_as_string (comp);
+		if (!rid || !*rid) {
 			remove_component (cbfile, uid, obj_data);
 			*object = NULL;
 		} else {
-			*old_object = get_object_string_from_fileobject (obj_data, recur_id);
-
-			remove_instance (cbfile, obj_data, recur_id);
-			if (comp)
-				*object = e_cal_component_get_as_string (comp);
+			remove_instance (cbfile, obj_data, rid);
+			*object = e_cal_component_get_as_string (obj_data->full_object);
 		}
 		break;
 	case CALOBJ_MOD_THISANDPRIOR :
 	case CALOBJ_MOD_THISANDFUTURE :
-		if (!recur_id || !*recur_id)
+		if (!rid || !*rid)
 			return GNOME_Evolution_Calendar_ObjectNotFound;
 
 		*old_object = e_cal_component_get_as_string (comp);
@@ -2082,12 +2084,12 @@ e_cal_backend_file_remove_object (ECalBackendSync *backend, EDataCal *cal,
 		priv->comp = g_list_remove (priv->comp, comp);
 
 		e_cal_util_remove_instances (e_cal_component_get_icalcomponent (comp),
-					     icaltime_from_string (recur_id), mod);
+					     icaltime_from_string (rid), mod);
 
 		/* now remove all detached instances */
 		rrdata.cbfile = cbfile;
 		rrdata.obj_data = obj_data;
-		rrdata.rid = recur_id;
+		rrdata.rid = rid;
 		rrdata.mod = mod;
 		g_hash_table_foreach_remove (obj_data->recurrences, (GHRFunc) remove_object_instance_cb, &rrdata);
 
@@ -2387,6 +2389,7 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 			break;
 		case ICAL_METHOD_CANCEL:
 			if (cancel_received_object (cbfile, subcomp)) {
+				ECalComponentId *id;
 				object = (char *) icalcomponent_as_ical_string (subcomp);
 				obj_data = g_hash_table_lookup (priv->comp_uid_hash, uid);
 				if (obj_data)
@@ -2394,11 +2397,14 @@ e_cal_backend_file_receive_objects (ECalBackendSync *backend, EDataCal *cal, con
 				else
 					old_object = NULL;
 
-				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), uid, old_object, object);
+				id = e_cal_component_get_id (comp);
+
+				e_cal_backend_notify_object_removed (E_CAL_BACKEND (backend), id, old_object, object);
 
 				/* remove the component from the toplevel VCALENDAR */
 				icalcomponent_remove_component (toplevel_comp, subcomp);
 				icalcomponent_free (subcomp);
+				e_cal_component_free_id (id);
 
 				g_free (old_object);
 			}
