@@ -76,6 +76,8 @@ struct _ECalBackendHttpPrivate {
 static void e_cal_backend_http_dispose (GObject *object);
 static void e_cal_backend_http_finalize (GObject *object);
 static gboolean begin_retrieval_cb (ECalBackendHttp *cbhttp);
+static ECalBackendSyncStatus
+e_cal_backend_http_add_timezone (ECalBackendSync *backend, EDataCal *cal, const char *tzobj);
 
 static ECalBackendSyncClass *parent_class;
 
@@ -200,8 +202,13 @@ notify_and_remove_from_cache (gpointer key, gpointer value, gpointer user_data)
 	const char *uid = key;
 	const char *calobj = value;
 	ECalBackendHttp *cbhttp = E_CAL_BACKEND_HTTP (user_data);
+	ECalComponent *comp = e_cal_component_new_from_string (calobj);
+	ECalComponentId *id = e_cal_component_get_id (comp);
 
-	e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbhttp), uid, calobj, NULL);
+	e_cal_backend_notify_object_removed (E_CAL_BACKEND (cbhttp), id, calobj, NULL);
+
+	e_cal_component_free_id (id);
+	g_object_unref (comp);
 
 	return TRUE;
 }
@@ -491,12 +498,18 @@ e_cal_backend_http_open (ECalBackendSync *backend, EDataCal *cal, gboolean only_
 	if (!priv->cache) {
 		priv->cache = e_cal_backend_cache_new (e_cal_backend_get_uri (E_CAL_BACKEND (backend)));
 
+
 		if (!priv->cache) {
 			e_cal_backend_notify_error (E_CAL_BACKEND(cbhttp), _("Could not create cache file"));
 			return GNOME_Evolution_Calendar_OtherError;	
 		}
-		if (priv->mode == CAL_MODE_LOCAL)
+		
+		e_cal_backend_cache_put_default_timezone (priv->cache, priv->default_zone);
+		e_cal_backend_http_add_timezone (backend, cal, (const char *) icaltimezone_get_tzid (priv->default_zone));
+	
+		if (priv->mode == CAL_MODE_LOCAL) 
 			return GNOME_Evolution_Calendar_Success;
+		
 
 		g_idle_add ((GSourceFunc) begin_retrieval_cb, cbhttp);
 	}
@@ -717,7 +730,9 @@ e_cal_backend_http_set_default_timezone (ECalBackendSync *backend, EDataCal *cal
 	cbhttp = E_CAL_BACKEND_HTTP (backend);
 	priv = cbhttp->priv;
 	
-	/* FIXME */
+	/* Set the default timezone to it. */
+	priv->default_zone = icaltimezone_get_builtin_timezone_from_tzid (tzid);
+	
 	return GNOME_Evolution_Calendar_Success;
 }
 
