@@ -9,7 +9,7 @@
  *    Dan Winship <danw@ximian.com>
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -17,7 +17,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -28,7 +28,6 @@
 #endif
 
 #include <sys/stat.h>
-#include <sys/uio.h>
 #include <unistd.h>
 #include <errno.h>
 #include <string.h>
@@ -36,6 +35,7 @@
 
 #include "camel-groupwise-summary.h"
 #include "camel-file-utils.h"
+#include <camel/camel-folder.h>
 
 #define CAMEL_GW_SUMMARY_VERSION (1)
 
@@ -48,7 +48,7 @@ static CamelMessageInfo *gw_message_info_load (CamelFolderSummary *s, FILE *in)
 static int gw_message_info_save (CamelFolderSummary *s, FILE *out, CamelMessageInfo *info) ;
 static CamelMessageContentInfo * gw_content_info_load (CamelFolderSummary *s, FILE *in) ;
 static int gw_content_info_save (CamelFolderSummary *s, FILE *out, CamelMessageContentInfo *info) ;
-		
+static gboolean gw_info_set_flags(CamelMessageInfo *info, guint32 flags, guint32 set);		
 
 static void camel_groupwise_summary_class_init (CamelGroupwiseSummaryClass *klass);
 static void camel_groupwise_summary_init       (CamelGroupwiseSummary *obj);
@@ -108,6 +108,7 @@ camel_groupwise_summary_class_init (CamelGroupwiseSummaryClass *klass)
 	cfs_class->message_info_save = gw_message_info_save;
 	cfs_class->content_info_load = gw_content_info_load;
 	cfs_class->content_info_save = gw_content_info_save;
+	cfs_class->info_set_flags = gw_info_set_flags;
 }
 
 
@@ -161,6 +162,8 @@ gw_summary_header_load (CamelFolderSummary *s, FILE *in)
 			|| camel_file_util_decode_fixed_int32(in, &ims->validity) == -1)
 		return -1;
 	
+	if (camel_file_util_decode_string (in, &ims->time_string) == -1)
+		return -1;
 	return 0 ;
 }
 
@@ -174,8 +177,9 @@ gw_summary_header_save (CamelFolderSummary *s, FILE *out)
 		return -1;
 
 	camel_file_util_encode_fixed_int32(out, CAMEL_GW_SUMMARY_VERSION);
+	camel_file_util_encode_fixed_int32(out, ims->validity);
+	return camel_file_util_encode_string (out, ims->time_string);
 
-	return camel_file_util_encode_fixed_int32(out, ims->validity);
 
 }
 
@@ -233,6 +237,48 @@ gw_content_info_save (CamelFolderSummary *s, FILE *out,
 		return fputc (0, out);
 }
 
+static gboolean
+gw_info_set_flags (CamelMessageInfo *info, guint32 flags, guint32 set)
+{
+	guint32 old;
+	CamelMessageInfoBase *mi = (CamelMessageInfoBase *)info;
+
+	/* TODO: locking? */
+
+	old = mi->flags;
+	/* we don't set flags which aren't appropriate for the folder*/
+	if ((set == (CAMEL_MESSAGE_JUNK|CAMEL_MESSAGE_JUNK_LEARN|CAMEL_MESSAGE_SEEN)) && (old & CAMEL_GW_MESSAGE_JUNK))
+		return FALSE;
+	
+	mi->flags = (old & ~flags) | (set & flags);
+	if (old != mi->flags) {
+		mi->flags |= CAMEL_MESSAGE_FOLDER_FLAGGED;
+		if (mi->summary)
+			camel_folder_summary_touch(mi->summary);
+	}
+	/* This is a hack, we are using CAMEL_MESSAGE_JUNK justo to hide the item
+	 * we make sure this doesn't have any side effects*/
+	
+	if ((set == CAMEL_MESSAGE_JUNK_LEARN) && (old & CAMEL_GW_MESSAGE_JUNK)) {
+		mi->flags |= CAMEL_MESSAGE_JUNK;
+		if (mi->summary) {
+			camel_folder_summary_touch(mi->summary);
+		}
+
+	} else	if ((old & ~CAMEL_MESSAGE_SYSTEM_MASK) == (mi->flags & ~CAMEL_MESSAGE_SYSTEM_MASK)) 
+		return FALSE;
+
+	if (mi->summary && mi->summary->folder && mi->uid) {
+		CamelFolderChangeInfo *changes = camel_folder_change_info_new();
+
+		camel_folder_change_info_change_uid(changes, camel_message_info_uid(info));
+		camel_object_trigger_event(mi->summary->folder, "folder_changed", changes);
+		camel_folder_change_info_free(changes);
+	}
+
+	return TRUE;
+
+}
 
 
 void
