@@ -3,7 +3,7 @@
 /* Copyright (C) 2002-2004 Novell, Inc.
  *
  * This program is free software; you can redistribute it and/or
- * modify it under the terms of version 2 of the GNU General Public
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -11,7 +11,7 @@
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * General Public License for more details.
  *
- * You should have received a copy of the GNU General Public
+ * You should have received a copy of the GNU Lesser General Public
  * License along with this program; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
@@ -39,6 +39,10 @@
 #include "e2k-uri.h"
 #include "e2k-utils.h"
 #include "exchange-hierarchy-foreign.h"
+
+/* This is an ugly hack to avoid API break */
+/* Added for get_authtype */
+#include "exchange-esource.h"
 #include <libedataserverui/e-passwords.h>
 
 #include <libgnome/gnome-util.h>
@@ -156,7 +160,7 @@ init (GObject *object)
 	account->priv->folders = g_hash_table_new (g_str_hash, g_str_equal);
 	account->priv->fresh_folders = NULL;
 	account->priv->discover_data_lock = g_mutex_new ();
-	account->priv->account_online = TRUE;
+	account->priv->account_online = FALSE;
 	account->priv->nt_domain = NULL;
 	account->priv->fsize = exchange_folder_size_new ();
 }
@@ -337,7 +341,7 @@ exchange_account_rescan_tree (ExchangeAccount *account)
 				     toplevel);
 
 		exchange_hierarchy_scan_subtree (account->priv->hierarchies->pdata[i],
-						toplevel, FALSE);
+						toplevel, !account->priv->account_online);
 		exchange_hierarchy_rescan (account->priv->hierarchies->pdata[i]);
 	}
 }
@@ -353,49 +357,56 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 	int table_updated = 0;
 	const char *permanent_uri =
 		e_folder_exchange_get_permanent_uri (folder);
+	char *key;
 
 	/* This makes the cleanup easier. We just unref it each time
 	 * we find it in account->priv->folders.
 	 */
-	if (!g_hash_table_lookup (account->priv->folders, 
-				e_folder_exchange_get_path (folder))) {
+	d(printf ("adding folder : %s\n", e_folder_get_name (folder)));
+	key = (char *) e_folder_exchange_get_path (folder);
+	if (!g_hash_table_lookup (account->priv->folders, key)) {
 		/* Avoid dupilcations since the user could add a folder as
 		  favorite even though it is already marked as favorite */
 		g_object_ref (folder);
 		g_hash_table_insert (account->priv->folders,
-				     (char *)e_folder_exchange_get_path (folder),
+				     key,
 				     folder);
+		d(printf ("added path : %s\n", key));
 		table_updated = 1;
 	}
 
 	if (account->priv->fresh_folders) {
 		g_object_ref (folder);
 		g_hash_table_insert (account->priv->fresh_folders,
-				     (char *)e_folder_exchange_get_path (folder),
+				     key,
 				     folder);
 	}	
 	
-	if (!g_hash_table_lookup (account->priv->folders, 
-				e_folder_get_physical_uri (folder))) {
+	key = (char *) e_folder_get_physical_uri (folder);
+	if (!g_hash_table_lookup (account->priv->folders, key)) {
 		/* Avoid dupilcations since the user could add a folder as
 		  favorite even though it is already marked as favorite */
 		g_object_ref (folder);
 		g_hash_table_insert (account->priv->folders,
-				     (char *)e_folder_get_physical_uri (folder),
+				     key,
 				     folder);
+		d(printf ("added phy path : %s\n", key));
 		table_updated = 1;
 	}
-	if (!g_hash_table_lookup (account->priv->folders, 
-				e_folder_exchange_get_internal_uri (folder))) {
+
+	key = (char *) e_folder_exchange_get_internal_uri (folder);
+	if (!g_hash_table_lookup (account->priv->folders, key)) {
 		/* The internal_uri for public folders and favorites folder 
 		   is same !!! Without this check the folder value could 
 		   overwrite the previously added folder. */
 		g_object_ref (folder);
 		g_hash_table_insert (account->priv->folders,
-				     (char *)e_folder_exchange_get_internal_uri (folder),
+				     key,
 				     folder);
+		d(printf ("added int path : %s\n", key));
 		table_updated = 1;
 	}
+
 	if (permanent_uri && (!g_hash_table_lookup (account->priv->folders, 
 					permanent_uri))) {
 		g_object_ref (folder);
@@ -404,7 +415,7 @@ hierarchy_new_folder (ExchangeHierarchy *hier, EFolder *folder,
 				     folder);
 		table_updated = 1;
 	}
-
+	
 	if (table_updated)
 	{
 		g_hash_table_insert (account->priv->hierarchies_by_folder, 
@@ -501,12 +512,24 @@ exchange_account_create_folder (ExchangeAccount *account,
 	return exchange_hierarchy_create_folder (hier, parent, path, type);
 }
 
+static gboolean
+check_if_sf (gpointer key, gpointer value, gpointer user_data)
+{
+	char *sf_href = (char *)value;
+	char *int_uri = (char *)user_data;
+
+	if (!strcmp (sf_href, int_uri))
+		return TRUE; /* Quit calling the callback */
+
+	return FALSE; /* Continue calling the callback till end of table */
+}
+
 ExchangeAccountFolderResult
 exchange_account_remove_folder (ExchangeAccount *account, const char *path)
 {
 	ExchangeHierarchy *hier;
 	EFolder *folder;
-	const char *name;
+	const char *int_uri;
 
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), 
 				EXCHANGE_ACCOUNT_FOLDER_GENERIC_ERROR);
@@ -514,9 +537,12 @@ exchange_account_remove_folder (ExchangeAccount *account, const char *path)
 	if (!get_folder (account, path, &folder, &hier))
 		return EXCHANGE_ACCOUNT_FOLDER_DOES_NOT_EXIST;
 	
-	name = e_folder_get_name (folder);
-	if (exchange_account_get_standard_uri (account, name))
+	int_uri = e_folder_exchange_get_internal_uri (folder);
+
+	if (g_hash_table_find (account->priv->standard_uris, 
+					check_if_sf, int_uri)) {
 		return EXCHANGE_ACCOUNT_FOLDER_UNSUPPORTED_OPERATION;
+	}
 
 	return exchange_hierarchy_remove_folder (hier, folder);
 }
@@ -617,7 +643,7 @@ get_hierarchy_for (ExchangeAccount *account, E2kGlobalCatalogEntry *entry)
 					  entry->display_name);
 	source = g_strdup_printf ("exchange://%s@%s/", entry->mailbox,
 				  account->exchange_server);
-	physical_uri_prefix = g_strdup_printf ("exchange://%s/%s",
+	physical_uri_prefix = g_strdup_printf ("exchange://%s/;%s",
 					       account->priv->uri_authority,
 					       entry->email);
 	internal_uri_prefix = exchange_account_get_foreign_uri (account, entry,
@@ -907,6 +933,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, ExchangeAccountResult
 
 	// SURF : if (exchange_component_is_interactive (global_exchange_component)) {
 		if (!password) {
+	/*
 			char *prompt;
 
 			prompt = g_strdup_printf (_("Enter password for %s"),
@@ -925,6 +952,7 @@ get_password (ExchangeAccount *account, E2kAutoconfig *ac, ExchangeAccountResult
 				account->priv->account->source->save_passwd = remember;
 			}
 			g_free (prompt);
+	*/
 		} 
 		else if (!account->priv->account->source->save_passwd) {
 			/* get_password returns the password cached but user has not 
@@ -1165,12 +1193,14 @@ exchange_account_set_offline (ExchangeAccount *account)
 		
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
 
+	g_mutex_lock (account->priv->connect_lock);
 	if (account->priv->ctx) {
 		g_object_unref (account->priv->ctx);
 		account->priv->ctx = NULL;
 	}
 
 	account->priv->account_online = FALSE;
+	g_mutex_unlock (account->priv->connect_lock);
 	return TRUE;
 }
 
@@ -1188,19 +1218,15 @@ exchange_account_set_offline (ExchangeAccount *account)
 gboolean
 exchange_account_set_online (ExchangeAccount *account)
 {
-	E2kContext *ctx;
-	ExchangeAccountResult result;
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), FALSE);
 
-	if (!account->priv->account_online) {
-		ctx = exchange_account_connect (account, NULL, &result); /* error not handled. */
-		return ctx ? TRUE : FALSE;
-	} else {
-		return TRUE;
-	}
+	g_mutex_lock (account->priv->connect_lock);
+	account->priv->account_online = TRUE;
+	g_mutex_unlock (account->priv->connect_lock);
+	
+	return TRUE;
 }
-#if 0
-SURF :
+
 /**
  * exchange_account_is_offline:
  * @account: an #ExchangeAccount
@@ -1210,18 +1236,9 @@ SURF :
 void
 exchange_account_is_offline (ExchangeAccount *account, int *state)
 {
-	*state = UNSUPPORTED_MODE;
-
 	g_return_if_fail (EXCHANGE_IS_ACCOUNT (account));
-
-	exchange_component_is_offline (global_exchange_component, state);
-}
-#endif
-void
-exchange_account_is_offline (ExchangeAccount *account, int *state)
-{
-	// SURF : Dummy
-	*state = ONLINE_MODE;
+	
+	*state = account->priv->account_online ? ONLINE_MODE : OFFLINE_MODE;
 }
 
 // SURF : Picked this from gal/util/e-util.c
@@ -1262,8 +1279,12 @@ setup_account_hierarchies (ExchangeAccount *account)
 	if (offline == UNSUPPORTED_MODE)
 		return FALSE;
 
+	/* Check if folder hierarchies are already setup. */
+	if (account->priv->hierarchies->len > 0)
+		goto hierarchies_created;
+
 	/* Set up Personal Folders hierarchy */
-	phys_uri_prefix = g_strdup_printf ("exchange://%s/personal",
+	phys_uri_prefix = g_strdup_printf ("exchange://%s/;personal",
 					   account->priv->uri_authority);
 	hier = exchange_hierarchy_webdav_new (account,
 					      EXCHANGE_HIERARCHY_PERSONAL,
@@ -1276,10 +1297,9 @@ setup_account_hierarchies (ExchangeAccount *account)
 					      TRUE);
 	setup_hierarchy (account, hier);
 	g_free (phys_uri_prefix);
-	personal_hier = hier;
 
 	/* Favorite Public Folders */
-	phys_uri_prefix = g_strdup_printf ("exchange://%s/favorites",
+	phys_uri_prefix = g_strdup_printf ("exchange://%s/;favorites",
 					   account->priv->uri_authority);
 	hier = exchange_hierarchy_favorites_new (account,
 						 _("Favorite Public Folders"),
@@ -1294,7 +1314,7 @@ setup_account_hierarchies (ExchangeAccount *account)
 	account->priv->favorites_hierarchy = hier;
 
 	/* Public Folders */
-	phys_uri_prefix = g_strdup_printf ("exchange://%s/public",
+	phys_uri_prefix = g_strdup_printf ("exchange://%s/;public",
 					   account->priv->uri_authority);
 	hier = exchange_hierarchy_webdav_new (account,
 					      EXCHANGE_HIERARCHY_PUBLIC,
@@ -1336,10 +1356,16 @@ setup_account_hierarchies (ExchangeAccount *account)
 		closedir (d);
 	}
 
+hierarchies_created:
+	
 	/* Scan the personal and favorite folders so we can resolve references
 	 * to the Calendar, Contacts, etc even if the tree isn't
 	 * opened.
 	 */
+
+	/* Assuming the first element being personal hierarchy. */
+	personal_hier = account->priv->hierarchies->pdata[0];
+	
 	fresult = exchange_hierarchy_scan_subtree (personal_hier,
 						   personal_hier->toplevel,
 						   (offline == OFFLINE_MODE));
@@ -1415,7 +1441,6 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	}
 
 	account->priv->connecting = TRUE;
-	g_mutex_unlock (account->priv->connect_lock);
 
 	if (account->priv->windows_domain)
 		user_name = g_strdup_printf ("%s\\%s", account->priv->windows_domain, account->priv->username);
@@ -1431,12 +1456,20 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	e2k_autoconfig_set_gc_server (ac, account->priv->ad_server,
 				      account->priv->ad_limit);
 
+	if (!pword) {
+		g_mutex_unlock (account->priv->connect_lock);
+		return NULL;
+	}
  try_password_again:
 	if (!pword) {
+/*
 		if (!get_password (account, ac, *info_result)) {
 			account->priv->connecting = FALSE;
 			return NULL;
 		}
+*/
+	} else {
+		e2k_autoconfig_set_password (ac, g_strdup (pword));
 	}
 
  try_connect_again:
@@ -1452,6 +1485,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 		if ( is_password_expired (account, ac)) {
 			*info_result = EXCHANGE_ACCOUNT_PASSWORD_EXPIRED;
 			account->priv->connecting = FALSE;
+			g_mutex_unlock (account->priv->connect_lock);
 			return NULL;
 /*
 			old_password = exchange_account_get_password (account);
@@ -1486,6 +1520,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 				goto try_password_again;
 			e2k_autoconfig_free (ac);
 			account->priv->connecting = FALSE;
+			g_mutex_unlock (account->priv->connect_lock);
 			return NULL;
 
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_DOMAIN:
@@ -1494,6 +1529,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 				goto try_password_again;
 			e2k_autoconfig_free (ac);
 			account->priv->connecting = FALSE;
+			g_mutex_unlock (account->priv->connect_lock);
 			return NULL;
 
 		case E2K_AUTOCONFIG_AUTH_ERROR_TRY_NTLM:
@@ -1546,12 +1582,13 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 			*info_result = EXCHANGE_ACCOUNT_CONNECT_ERROR;
 			break;
 		case E2K_AUTOCONFIG_CANCELLED:
-			return NULL;
+			break;
 		default:
 			*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
 			break;
 		}
 
+		g_mutex_unlock (account->priv->connect_lock);
 		return NULL;
 	}
 
@@ -1567,6 +1604,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 	if (!E2K_HTTP_STATUS_IS_SUCCESSFUL (status)) {
 		account->priv->connecting = FALSE;
 		*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
+		g_mutex_unlock (account->priv->connect_lock);
 		return NULL; /* FIXME: what error has happened? */
 	}
 
@@ -1588,6 +1626,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 
 	if (!setup_account_hierarchies (account)) {
 		*info_result = EXCHANGE_ACCOUNT_UNKNOWN_ERROR;
+		g_mutex_unlock (account->priv->connect_lock);
 		return NULL; /* FIXME: what error has happened? */
 	}
 
@@ -1632,6 +1671,7 @@ exchange_account_connect (ExchangeAccount *account, const char *pword,
 			  G_CALLBACK (context_redirect), account);
 
 	g_signal_emit (account, signals[CONNECTED], 0, account->priv->ctx);
+	g_mutex_unlock (account->priv->connect_lock);
 	return account->priv->ctx;
 }
 
@@ -1810,6 +1850,9 @@ exchange_account_get_folder (ExchangeAccount *account,
 {
 	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
 
+	if (!path_or_uri)
+		return NULL;
+
 	return g_hash_table_lookup (account->priv->folders, path_or_uri);
 }
 
@@ -1982,6 +2025,20 @@ exchange_account_folder_size_get_model (ExchangeAccount *account)
 	return exchange_folder_size_get_model (account->priv->fsize);
 }
 
+char *
+exchange_account_get_authtype (ExchangeAccount *account)
+{
+	g_return_val_if_fail (EXCHANGE_IS_ACCOUNT (account), NULL);
+
+	if (account->priv->auth_pref == E2K_AUTOCONFIG_USE_BASIC)
+		return g_strdup ("Basic");
+	else if (account->priv->auth_pref == E2K_AUTOCONFIG_USE_NTLM)
+		return g_strdup ("NTLM");
+
+	return NULL;
+}
+
+
 /**
  * exchange_account_new:
  * @adata: an #EAccount
@@ -1995,7 +2052,7 @@ ExchangeAccount *
 exchange_account_new (EAccountList *account_list, EAccount *adata)
 {
 	ExchangeAccount *account;
-	char *enc_user, *mailbox;
+	char *enc_user, *mailbox, *old_uri_authority;
 	const char *param, *proto="http", *owa_path, *pf_server, *owa_url; 
 	const char *passwd_exp_warn_period, *offline_sync;
 	E2kUri *uri;
@@ -2018,8 +2075,8 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 	account->storage_dir = g_strdup_printf ("%s/.evolution/exchange/%s@%s",
 						g_get_home_dir (),
 						uri->user, uri->host);
-	account->account_filename = strrchr (account->storage_dir, '/') + 1;
-	e_filename_make_safe (account->account_filename);
+	/*account->account_filename = strrchr (account->storage_dir, '/') + 1;
+	e_filename_make_safe (account->account_filename); */
 
 	/* Identity info */
 	account->priv->identity_name = g_strdup (adata->id->name);
@@ -2027,16 +2084,25 @@ exchange_account_new (EAccountList *account_list, EAccount *adata)
 
 	/* URI, etc, info */
 	enc_user = e2k_uri_encode (uri->user, FALSE, "@/;:");
-	account->priv->uri_authority = g_strdup_printf ("%s@%s", enc_user,
+	old_uri_authority = g_strdup_printf ("%s@%s", enc_user,
 							uri->host);
+	if (uri->authmech)
+		account->priv->uri_authority = g_strdup_printf ("%s;auth=%s@%s", enc_user,
+								uri->authmech, uri->host);
+	else
+		account->priv->uri_authority = g_strdup_printf ("%s@%s", enc_user,
+								uri->host);
 	g_free (enc_user);
 
+	account->account_filename = account->priv->uri_authority;
+
 	account->priv->source_uri = g_strdup_printf ("exchange://%s/", account->priv->uri_authority);
 
 	/* Backword compatibility; FIXME, we should just migrate the
 	 * password from this to source_uri.
 	 */
-	account->priv->password_key = g_strdup_printf ("exchange://%s", account->priv->uri_authority);
+	account->priv->password_key = g_strdup_printf ("exchange://%s", old_uri_authority);
+	g_free (old_uri_authority);
 
 	account->priv->username = g_strdup (uri->user);
 	if (uri->domain)
